@@ -2,7 +2,7 @@ import pandas as pd
 import pywencai as wc
 import tushare as ts
 import os
-from kit import num_kit
+from kit import num_kit, path_kit
 
 '''
 数据函数
@@ -10,24 +10,28 @@ from kit import num_kit
 
 
 # 导入工程
-class dataset_loader:
+class base_data_loader:
     """
     数据加载器
     通过问财获得股票列表
     通过tushare获得股票数据
     """
 
-    def __init__(self, question, start, end, token):
-        self.question = question
-        self.start = start
-        self.end = end
-        self.token = token
-        self.symbols_list = None  # 股票列表
+    def __init__(self, path="../data/", module="data/", file_name="base_data.csv"):
+        self.path = path
+        self.module = module
+        self.file_name = file_name
+        self.module_path = path + module
+        self.question = None
+        self.start = None
+        self.end = None
+        self.token = None
         self.symbol_index = dict()  # 股票时间索引
         self.filter = set()  # 股票的过滤集
         self.data = pd.DataFrame()
+        path_kit.create_dir(self.module_path)
 
-    def __daily_data(self, symbol, start, end):
+    def __daily_data(self, symbol, start, end, token):
         """
         获取日线数据
         :param symbol:
@@ -35,7 +39,7 @@ class dataset_loader:
         :param end:
         :return:
         """
-        api = ts.pro_api(self.token)
+        api = ts.pro_api(token)
         df = ts.pro_bar(
             ts_code=symbol,
             api=api,
@@ -49,7 +53,7 @@ class dataset_loader:
         )
         return df[::-1]
 
-    def __daily(self, start_date, end_date, symbols=[]):
+    def __daily(self, start_date, end_date, token, symbols: list):
         """
         获取日线数据
         :param start_date:
@@ -61,7 +65,7 @@ class dataset_loader:
         if len(symbols) == 0:
             return pd.DataFrame()
         for symbol in symbols:
-            df = self.__daily_data(symbol, start_date, end_date)
+            df = self.__daily_data(symbol, start_date, end_date, token)
             result = pd.concat([result, df])
         return result
 
@@ -74,7 +78,7 @@ class dataset_loader:
         symbols_set = set(symbols)
         self.filter.update(symbols_set)
 
-    def __get_symbols_by_wc(self, question, columns=[]):
+    def __get_symbols_by_wc(self, question, columns: list):
         """
         股票列表
         通过问财获得股票列表
@@ -87,38 +91,40 @@ class dataset_loader:
             data = data[~data['股票代码'].isin(self.filter)]
             data['trade_date'] = i
             result = pd.concat([result, data])
-        self.symbols_list = result
         return result
 
-    def get_data(self, data_path='../data/data.csv'):
+    def get_data(self, question, start, end, token):
         """
         获取总数据集
         优先在本地读取，如果本地没有从互联网获取
+        :param token:
+        :param end:
+        :param start:
+        :param question:
         :param data_path: 默认的数据集路径
         :return:
         """
-        if os.path.exists(data_path):
+
+        self.question = question
+        self.start = start
+        self.end = end
+
+        if os.path.exists(self.module_path + self.file_name):
             print("读取本地数据集")
-            self.data = pd.read_csv(data_path)
+            self.data = pd.read_csv(self.module_path + self.file_name)
         else:
             print("从互联网获取数据集")
             symbols_list = self.__get_symbols_by_wc(self.question, columns=['股票代码'])
             print("开始遍历")
             for index, symbol in enumerate(symbols_list['股票代码'].unique()):
                 print("数据进度百分比:%s" % (index / len(symbols_list['股票代码'].unique()) * 100), end='\r', flush=True)
-                # 获取股票代码的符合的年数据
-                # symbol_data = symbols_list[symbols_list['股票代码'] == symbol]
-                # # 获取股票代码的年数据的时间集合
-                # self.symbol_index.update({symbol: num_kit.date_split(symbol_data['trade_date'])})
-                self.data = pd.concat([self.data, self.__daily_data(symbol, self.start, self.end)])
-
-
+                self.data = pd.concat([self.data, self.__daily_data(symbol, self.start, self.end, token)])
         # 重置索引
         self.data = self.data.reset_index(drop=True)
         # 将日期转换为字符串
         self.data['trade_date'] = self.data['trade_date'].apply(lambda x: str(x))
-        # 将数据里的数字类型转换为float 保留两位小数
-        self.data = self.data.applymap(lambda x: round(float(x), 2) if isinstance(x, (int, float)) else x)
+        # 将数据里的数字类型转换为float
+        self.data = self.data.applymap(lambda x: float(x) if isinstance(x, (int, float)) else x)
         return self.data
 
     def observe(self, mlflow):
@@ -127,19 +133,22 @@ class dataset_loader:
         :return:
         """
         # 数据报告
-        mlflow.log_text("\n".join(self.__data_report()),"data_report" )
-        # 新增数据集
+        mlflow.log_text("\n".join(self.__data_report()), self.module + "data_report.txt")
+        # 获取每个股票的时间个数
+        mlflow.log_dict(self.data.groupby('ts_code')['trade_date'].count().to_dict(), self.module + "data_count.txt")
+        # 每个数据的缺失日期情况
+        mlflow.log_text(self.__row_nan_trade_date(), self.module + "data_trade_nan.txt")
 
-    def save(self, path='../data/', name='data.csv'):
+    def save(self, path=None, name=None):
         """
         保存数据集
         :param name:
         :param path:
         :return:
         """
-        if not os.path.exists(path):
-            os.mkdir(path)
-        self.data.to_csv(path+name, index=False, encoding='utf-8',)
+        file_path = self.module_path if path is None else path
+        file_name = self.file_name if name is None else name
+        self.data.to_csv(file_path + file_name, index=False, encoding='utf-8', )
 
     def __data_report(self):
         """
@@ -153,13 +162,27 @@ class dataset_loader:
         data.append("结束日期:%s" % self.end)
         data.append("数据总量:%s" % len(self.data))
         data.append("数据列数:%s" % len(self.data.columns))
-        data.append("数据列名:%s" % self.data.columns)
+        data.append("数据列名:%s" % self.data.columns.tolist())
         data.append("数据集缺失值:%s" % self.data.isnull().sum())
         return data
 
+    def __row_nan_trade_date(self):
+        text = ''
+        symbols = self.data["ts_code"].unique()
+        trades = self.data["trade_date"].unique()
+        for symbol in symbols:
+            trade_list = pd.DataFrame(trades)
+            trade_list.columns = ['trade_date']
+            trade_data = pd.merge(trade_list, self.data[self.data['ts_code'] == symbol], on='trade_date', how='left')
+            trade_data = trade_data[trade_data['ts_code'].isnull()]
+            if len(trade_data) != 0:
+                text = text + symbol + ','
+                text = text + ",".join(trade_data['trade_date'].astype('str').tolist()) + '\n'
+        return text
+
 
 # 特征工程
-class dataset_feature:
+class feature_data_loader:
     """
     数据特征工程
     1.添加特征
@@ -167,11 +190,14 @@ class dataset_feature:
     3.保存数据集
     """
 
-    def __init__(self, data):
-        self.base = data
+    def __init__(self, base_data=None, path="../data/", module="data/", file_name="feature_data.csv"):
+        self.data = base_data
         self.features_list = []
-        self.data = pd.DataFrame()
-
+        self.result = pd.DataFrame()
+        self.path = path
+        self.module = module
+        self.file_name = file_name
+        self.module_path = path + module
     def add_feature(self, feature):
         """
         添加特征
@@ -189,37 +215,47 @@ class dataset_feature:
         """
         pass
 
-    def save(self, path='../data/', name='dataset.csv'):
+    def save(self, path=None, name=None):
         """
         保存数据集
         :param name:
         :param path:
         :return:
         """
-        if not os.path.exists(path):
-            os.mkdir(path)
-        self.data.to_csv(path, index=False, encoding='utf-8')
+        file_path = self.module_path if path is None else path
+        file_name = self.file_name if name is None else name
+        self.result.to_csv(file_path + file_name, index=False, encoding='utf-8')
 
     def execute(self):
         """
         执行特征工程
         :return:
         """
-        symbol_list = self.base['ts_code'].unique()
+        symbol_list = self.data['ts_code'].unique()
+        columns = self.data.columns
         for symbol in symbol_list:
-            symbol_data = pd.DataFrame(self.base[self.base['ts_code'] == symbol])
+            symbol_data = pd.DataFrame(self.data[self.data['ts_code'] == symbol])
+            symbol_data.reset_index(drop=True, inplace=True)
             for func in self.features_list:
                 func(symbol_data)
-            self.data = pd.concat([self.data, symbol_data])
-        return self.data
+            # 将symbol_data 按照旧列和新列分成2个数据集
+            symbol_data_left = symbol_data[columns]
+            symbol_data_right = symbol_data[symbol_data.columns[~symbol_data.columns.isin(symbol_data_left.columns)]]
+            symbol_data_right.applymap(lambda x: round(float(x), 2) if isinstance(x, (int, float)) else x)
+            # 将新列数据集和旧列数据集合并
+            data = pd.merge(symbol_data_left, symbol_data_right, left_index=True, right_index=True)
+            self.result = pd.concat([self.result, data])
+        return self.result
 
 
 # 训练测试工程
-class dataset_train_test:
+class trains_data_loader:
 
-    def __init__(self, data, name):
-        self.base = data
-        self.name = name
+    def __init__(self, base_data=None, path="../data/", module="data/"):
+        self.base = base_data
+        self.path = path
+        self.module = module
+        self.module_path = path + module
         self.train_X = pd.DataFrame()
         self.train_y = pd.DataFrame()
         self.test_X = pd.DataFrame()
@@ -260,31 +296,54 @@ class dataset_train_test:
         return self.test_X, self.test_y
 
     def obverse(self, mlflow):
-        """
-        观察数据集
-        :return:
-        """
-        self.__data_report(self.train_X, self.train_y, mlflow)
+        mlflow.log_metric("train_label_1", len(self.train_X[self.train_X['flag'] == 1])/len(self.train_X) * 100)
+        mlflow.log_metric("train_label_0", len(self.train_X[self.train_X['flag'] == 1])/len(self.train_X) * 100)
+        mlflow.log_metric("train_label_-1", len(self.train_X[self.train_X['flag'] == 1])/len(self.train_X) * 100)
 
-    def __data_report(self, X, y, mlflow, name='train'):
-        data_X = X.copy()
-        data_y = y.copy()
-
-        mlflow.log_metric("train_X_count", len(data_X))
-        mlflow.log_metric("train_y_count", len(data_y))
-
-
-    def save(self, path='../data/'):
+    def save(self, path=None):
         """
         保存数据集
         :param path:
         :return:
         """
-        file_path = path + self.name + '/'
-        if not os.path.exists(file_path):
-            os.makedirs(file_path)
+        file_path = self.module_path if path is None else path
         self.train_X.to_csv(file_path + 'train_X.csv', index=False, encoding='utf-8')
         self.train_y.to_csv(file_path + 'train_y.csv', index=False, encoding='utf-8')
         self.test_X.to_csv(file_path + 'test_X.csv', index=False, encoding='utf-8')
         self.test_y.to_csv(file_path + 'test_y.csv', index=False, encoding='utf-8')
-        pass
+
+
+class backtrader_data_loader:
+
+    def __init__(self, path='../data/', csv_data='dataset.csv'):
+        self.path = path
+        # self.module = module
+        self.csv_data = csv_data
+        self.data = pd.DataFrame()
+
+    def get_data(self, start, end):
+        """
+        复权前的数据+ 复权后的指标
+        :param start:
+        :param end:
+        :return:
+        """
+        data = pd.read_csv(self.path + self.csv_data)
+        self.data = data[(data['trade_date'] > start) & (data['trade_date'] < end)]
+        self.data['open'] = round(self.data['open'] / self.data['adj_factor'], 2)
+        self.data['high'] = round(self.data['high'] / self.data['adj_factor'], 2)
+        self.data['low'] = round(self.data['low'] / self.data['adj_factor'], 2)
+        self.data['close'] = round(self.data['close'] / self.data['adj_factor'], 2)
+        self.data['amount'] = round(self.data['amount'] / self.data['adj_factor'], 2)
+        self.data.drop(['pre_close', 'change', 'pct_chg', 'flag'], axis=1, inplace=True)
+        self.data['trade_date'] = self.data['trade_date'].apply(lambda x: pd.to_datetime(x, format='%Y%m%d'))
+        self.data = self.data.rename(columns={'trade_date': 'datetime',
+                                              'vol': 'volume'
+                                              })
+        return self.data
+
+    def save(self, module, name):
+        path = self.path + module
+        if not os.path.exists(path):
+            os.mkdir(path)
+        self.data.to_csv(path + name, index=False, encoding='utf-8')
