@@ -11,3 +11,10 @@ def get_before_date(date, days=60):
     date = pd.to_datetime(date)
     date = date - pd.Timedelta(days=days)
     return date.strftime("%Y-%m-%d")
+
+def now(type="%Y%m%d%H%M%S"):
+    """
+    获取当前时间
+    :return:
+    """
+    return pd.to_datetime("now").strftime(type)
\ No newline at end of file
